package javax.naming.spi;

import javax.naming.CannotProceedException;
import javax.naming.NoInitialContextException;
import javax.naming.RefAddr;
import javax.naming.StringRefAddr;
import javax.naming.Referenceable;
import com.sun.naming.internal.FactoryEnumeration;
import com.sun.naming.internal.ResourceManager;
import java.util.Hashtable;
import javax.naming.Context;
import javax.naming.Name;
import java.net.MalformedURLException;
import javax.naming.Reference;
import javax.naming.NamingException;
import com.sun.naming.internal.VersionHelper;

public class NamingManager
{
    static final VersionHelper helper;
    private static ObjectFactoryBuilder object_factory_builder;
    private static final String defaultPkgPrefix = "com.sun.jndi.url";
    private static InitialContextFactoryBuilder initctx_factory_builder;
    public static final String CPE = "java.naming.spi.CannotProceedException";
    
    NamingManager() {
    }
    
    public static synchronized void setObjectFactoryBuilder(ObjectFactoryBuilder object_factory_builder) throws NamingException {
        if (NamingManager.object_factory_builder != null) {
            throw new IllegalStateException("ObjectFactoryBuilder already set");
        }
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkSetFactory();
        }
        NamingManager.object_factory_builder = object_factory_builder;
        return;
    }
    
    static synchronized ObjectFactoryBuilder getObjectFactoryBuilder() {
        return NamingManager.object_factory_builder;
    }
    
    static ObjectFactory getObjectFactoryFromReference(Reference reference, String s) throws IllegalAccessException, InstantiationException, MalformedURLException {
        Class<Object> clazz = null;
        try {
            clazz = NamingManager.helper.loadClass(s);
        }
        catch (ClassNotFoundException ex) {}
        String factoryClassLocation;
        if (clazz == null && (factoryClassLocation = reference.getFactoryClassLocation()) != null) {
            try {
                clazz = NamingManager.helper.loadClass(s, factoryClassLocation);
            }
            catch (ClassNotFoundException ex2) {}
        }
        return (clazz != null) ? ((ObjectFactory)clazz.newInstance()) : null;
    }
    
    private static Object createObjectFromFactories(Object o, Name name, Context ctx, Hashtable<?, ?> env) throws Exception {
        FactoryEnumeration factories = ResourceManager.getFactories("java.naming.factory.object", env, ctx);
        if (factories == null) {
            return null;
        }
        Object objectInstance;
        for (objectInstance = null; objectInstance == null && factories.hasMore(); objectInstance = ((ObjectFactory)factories.next()).getObjectInstance(o, name, ctx, env)) {}
        return objectInstance;
    }
    
    private static String getURLScheme(String s) {
        int index = s.indexOf(58);
        int index2 = s.indexOf(47);
        if (index > 0 && (index2 == -1 || index < index2)) {
            return s.substring(0, index);
        }
        return null;
    }
    
    public static Object getObjectInstance(Object o, Name name, Context context, Hashtable<?, ?> hashtable) throws Exception {
        ObjectFactoryBuilder objectFactoryBuilder = getObjectFactoryBuilder();
        if (objectFactoryBuilder != null) {
            return objectFactoryBuilder.createObjectFactory(o, hashtable).getObjectInstance(o, name, context, hashtable);
        }
        Reference reference = null;
        if ((o instanceof Reference)) {
            reference = (Reference)o;
        }
        else if ((o instanceof Referenceable)) {
            reference = ((Referenceable)(Referenceable)o).getReference();
        }
        if (reference != null) {
            String factoryClassName = reference.getFactoryClassName();
            if (factoryClassName != null) {
                ObjectFactory objectFactoryFromReference = getObjectFactoryFromReference(reference, factoryClassName);
                if (objectFactoryFromReference != null) {
                    return objectFactoryFromReference.getObjectInstance(reference, name, context, hashtable);
                }
                return o;
            }
            else {
                Object processURLAddrs = processURLAddrs(reference, name, context, hashtable);
                if (processURLAddrs != null) {
                    return processURLAddrs;
                }
            }
        }
        Object objectFromFactories = createObjectFromFactories(o, name, context, hashtable);
        return (objectFromFactories != null) ? objectFromFactories : o;
    }
    
    static Object processURLAddrs(Reference reference, Name name, Context context, Hashtable<?, ?> hashtable) throws NamingException {
        for (int i = 0; i < reference.size(); ++i) {
            RefAddr value = reference.get(i);
            if (value instanceof StringRefAddr && value.getType().equalsIgnoreCase("URL")) {
                Object processURL = processURL((Object)value.getContent(), name, context, hashtable);
                if (processURL != null) {
                    return processURL;
                }
            }
        }
        return null;
    }
    
    private static Object processURL(Object o, Name name, Context context, Hashtable<?, ?> hashtable) throws NamingException {
        if ((o instanceof String)) {
            String urlScheme = getURLScheme((String)o);
            if (urlScheme != null) {
                Object urlObject = getURLObject(urlScheme, o, name, context, hashtable);
                if (urlObject != null) {
                    return urlObject;
                }
            }
        }
        if ((o instanceof String[])) {
            String[] array = (String[])(String[])o;
            for (int i = 0; i < array.length; ++i) {
                String urlScheme2 = getURLScheme(array[i]);
                if (urlScheme2 != null) {
                    Object urlObject2 = getURLObject(urlScheme2, o, name, context, hashtable);
                    if (urlObject2 != null) {
                        return urlObject2;
                    }
                }
            }
        }
        return null;
    }
    
    static Context getContext(Object o, Name name, Context context, Hashtable<?, ?> hashtable) throws NamingException {
        if ((o instanceof Context)) {
            return (Context)o;
        }
        Object objectInstance;
        try {
            objectInstance = getObjectInstance(o, name, context, hashtable);
        }
        catch (NamingException ex) {
            throw ex;
        }
        catch (Exception rootCause) {
            NamingException ex2 = new NamingException();
            ex2.setRootCause(rootCause);
            throw ex2;
        }
        return (objectInstance instanceof Context) ? ((Context)objectInstance) : null;
    }
    
    static Resolver getResolver(Object o, Name name, Context context, Hashtable<?, ?> hashtable) throws NamingException {
        if ((o instanceof Resolver)) {
            return (Resolver)o;
        }
        Object objectInstance;
        try {
            objectInstance = getObjectInstance(o, name, context, hashtable);
        }
        catch (NamingException ex) {
            throw ex;
        }
        catch (Exception rootCause) {
            NamingException ex2 = new NamingException();
            ex2.setRootCause(rootCause);
            throw ex2;
        }
        return (objectInstance instanceof Resolver) ? ((Resolver)objectInstance) : null;
    }
    
    public static Context getURLContext(String s, Hashtable<?, ?> hashtable) throws NamingException {
        Object urlObject = getURLObject(s, null, null, null, hashtable);
        if ((urlObject instanceof Context)) {
            return (Context)urlObject;
        }
        return null;
    }
    
    private static Object getURLObject(String s, Object o, Name name, Context ctx, Hashtable<?, ?> env) throws NamingException {
        ObjectFactory objectFactory = (ObjectFactory)ResourceManager.getFactory("java.naming.factory.url.pkgs", env, ctx, "." + s + "." + s + "URLContextFactory", "com.sun.jndi.url");
        if (objectFactory == null) {
            return null;
        }
        try {
            return objectFactory.getObjectInstance(o, name, ctx, env);
        }
        catch (NamingException ex) {
            throw ex;
        }
        catch (Exception rootCause) {
            NamingException ex2 = new NamingException();
            ex2.setRootCause(rootCause);
            throw ex2;
        }
    }
    
    private static synchronized InitialContextFactoryBuilder getInitialContextFactoryBuilder() {
        return NamingManager.initctx_factory_builder;
    }
    
    public static Context getInitialContext(Hashtable<?, ?> hashtable) throws NamingException {
        InitialContextFactoryBuilder initialContextFactoryBuilder = getInitialContextFactoryBuilder();
        InitialContextFactory initialContextFactory;
        if (initialContextFactoryBuilder == null) {
            String s = (hashtable != null) ? ((String)hashtable.get("java.naming.factory.initial")) : null;
            if (s == null) {
                throw new NoInitialContextException("Need to specify class name in environment or system property, or as an applet parameter, or in an application resource file:  java.naming.factory.initial");
            }
            try {
                initialContextFactory = (InitialContextFactory)NamingManager.helper.loadClass(s).newInstance();
            }
            catch (Exception rootCause) {
                NoInitialContextException o = new NoInitialContextException("Cannot instantiate class: " + s);
                o.setRootCause(rootCause);
                throw o;
            }
        }
        else {
            initialContextFactory = initialContextFactoryBuilder.createInitialContextFactory(hashtable);
        }
        return initialContextFactory.getInitialContext(hashtable);
    }
    
    public static synchronized void setInitialContextFactoryBuilder(InitialContextFactoryBuilder initctx_factory_builder) throws NamingException {
        if (NamingManager.initctx_factory_builder != null) {
            throw new IllegalStateException("InitialContextFactoryBuilder already set");
        }
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkSetFactory();
        }
        NamingManager.initctx_factory_builder = initctx_factory_builder;
        return;
    }
    
    public static boolean hasInitialContextFactoryBuilder() {
        return getInitialContextFactoryBuilder() != null;
    }
    
    public static Context getContinuationContext(CannotProceedException value) throws NamingException {
        Hashtable<Object, Object> environment = value.getEnvironment();
        Hashtable hashtable;
        if (environment == null) {
            hashtable = new Hashtable(7);
        }
        else {
            hashtable = (Hashtable)environment.clone();
        }
        hashtable.put("java.naming.spi.CannotProceedException", value);
        return new ContinuationContext(value, hashtable).getTargetContext();
    }
    
    public static Object getStateToBind(Object o, Name name, Context ctx, Hashtable<?, ?> env) throws NamingException {
        FactoryEnumeration factories = ResourceManager.getFactories("java.naming.factory.state", env, ctx);
        if (factories == null) {
            return o;
        }
        Object stateToBind;
        for (stateToBind = null; stateToBind == null && factories.hasMore(); stateToBind = ((StateFactory)factories.next()).getStateToBind(o, name, ctx, env)) {}
        return (stateToBind != null) ? stateToBind : o;
    }
    
    static {
        helper = VersionHelper.getVersionHelper();
        NamingManager.object_factory_builder = null;
        NamingManager.initctx_factory_builder = null;
    }
}
